import abc
import os
import unittest

from integration.partial.integration_test_utils import IntegrationTestUtils


class BaseEpiTestSuite:

    class EpiTestSuite(unittest.TestCase):

        """
        the name passed to the Processor.py to define the component
        """
        EPI_COMPONENT_NAME = "Epidemiology"
        """
        the prefix of the directory created by thr Processor.py, e.g. EPI_2020-01-01
        """
        EPI_PROCESSOR_DIR = "EPI"

        @abc.abstractmethod
        def set_expected_values(self):
            pass

        def test_standard_run_input_status_success(self):
            status_file_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                            IntegrationTestUtils.TEST_JOB_DIR, "STATUS_SUCCESS")
            success_file_exists: bool = os.path.isfile(status_file_path)
            self.assertTrue(success_file_exists)


        def test_standard_run_input_all_stem_rust_files_produced(self):
            alpha_beta_gamma_root = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                                 IntegrationTestUtils.TEST_JOB_DIR,
                                                 "EastAfrica",
                                                 "StemRust",
                                                 "*psbeta0.004gamma0.00025alpha1.0")

            self.check_wildcard_exists_and_not_empty(alpha_beta_gamma_root, ".csv")
            self.check_wildcard_exists_and_not_empty(alpha_beta_gamma_root, "*comparison.png")
            self.check_wildcard_exists_and_not_empty(alpha_beta_gamma_root, "*progression.csv")
            self.check_wildcard_exists_and_not_empty(alpha_beta_gamma_root, ".png")
            self.check_wildcard_exists_and_not_empty(alpha_beta_gamma_root, ".tif")

            env_root = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                    IntegrationTestUtils.TEST_JOB_DIR,
                                    "EastAfrica",
                                    "StemRust",
                                    "infections_temp_config_Epidemiology")

            self.check_wildcard_exists_and_not_empty(env_root, "*env.csv")
            self.check_wildcard_exists_and_not_empty(env_root, "*env.png")
            self.check_wildcard_exists_and_not_empty(env_root, "*env.tif")

            data_input_root = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                           IntegrationTestUtils.TEST_JOB_DIR,
                                           "EastAfrica",
                                           "StemRust",
                                           "input_data",
                                           "data_input*")

            self.check_wildcard_exists_and_not_empty(data_input_root, "*deposition.csv")
            self.check_wildcard_exists_and_not_empty(data_input_root, "*environment.csv")
            self.check_wildcard_exists_and_not_empty(data_input_root, "*host.csv")

        def test_standard_run_input_all_plotting_files_produced(self):
            input_csvs_root = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                           IntegrationTestUtils.TEST_JOB_DIR,
                                           "plotting",
                                           "input_nc",
                                           "*stem*")

            self.check_wildcard_exists_and_not_empty(input_csvs_root, ".nc")

            images_root = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                       IntegrationTestUtils.TEST_JOB_DIR,
                                       "plotting",
                                       "images",
                                       "*stem*")

            self.check_wildcard_exists_and_not_empty(images_root, "png")


        def check_wildcard_exists_and_not_empty(self, root: str, extension: str):
            file_exists: bool = IntegrationTestUtils.check_wildcard_exists_and_not_empty(root + extension)
            self.assertTrue(file_exists)
